@@ -12,15 +12,13 @@ use std::fs::write;
 ///
 /// # Terms
 /// - temp_dir: Whatever the OS temp directory is
-/// - executed_filename: Last executed filename so we can restart state if necessary
-/// - functions: will rip out anything inside a fn() and put it in the outer scope
+/// - filename: Last executed filename so we can restart state if necessary
 /// - cells: VS Code name for a notebook cell
 
 #[derive(Debug)]
 pub struct Program {
     pub temp_dir: String,
-    pub executed_filename: String,
-    pub functions: String,
+    pub filename: String,
     pub cells: HashMap<u32, Cell>,
 }
 
@@ -30,14 +28,12 @@ pub struct Program {
 /// - fragment: unique id of the cell that executed
 /// - index: current index by order in VS Code
 /// - contents: the source code from inside the cell
-/// - executing: if true this cell is where the request came from,
 /// used to determine what output to return to caller
 #[derive(Debug)]
 pub struct Cell {
     pub fragment: u32,
     pub index: u32,
     pub contents: String,
-    pub executing: bool,
 }
 
 impl Program {
@@ -50,10 +46,11 @@ impl Program {
             .to_string();
         temp_file = temp_file;
 
+        println!("Ctrl+Click to view rust file: {}/main.rs", temp_file);
+        println!("Ctrl+Click to view cargo file: {}/Cargo.toml", temp_file);
         Program {
             temp_dir: temp_file,
-            executed_filename: "".to_string(),
-            functions: "".to_string(),
+            filename: String::new(),
             cells: HashMap::new(),
         }
     }
@@ -62,23 +59,19 @@ impl Program {
     /// Contents Use clone as we need to retain
     /// a pointer to data that will stick around after the request has finished
     pub fn create_cell(&mut self, cr: &CodeRequest) {
-        println!("Creating cell with fragment id: {}", cr.fragment);
-        let first_cell = Cell {
+        let cell = Cell {
             contents: cr.contents.clone(),
-            executing: true,
             fragment: cr.fragment,
             index: cr.index,
         };
-        self.cells.insert(cr.fragment, first_cell);
+        self.cells.insert(cr.fragment, cell);
     }
 
     /// As the cells are implemented with a HashMap this is a fast lookup,
     /// Only updates what's required
     pub fn update_cell(&mut self, cr: &CodeRequest) {
-        println!("Updating cell with fragment id: {}", cr.fragment);
         if let Some(cell) = self.cells.get_mut(&cr.fragment) {
             cell.contents = cr.contents.clone();
-            cell.executing = true;
             cell.index = cr.index;
         }
     }
@@ -87,33 +80,59 @@ impl Program {
     /// `rustkernel-start` and `rustkernel-end` to determine which cell's was executing
     // so we return only that output.
     pub fn write_to_file(&mut self, fragment: u32) {
-        // Sort based on current index's in VS Code
+        // Sort based on current cell indices in VS Code
         let mut cells_vec: Vec<&Cell> = self.cells.iter().map(|(_, cell)| (cell)).collect();
         cells_vec.sort_by(|a, b| a.index.cmp(&b.index));
 
         // Write the file output
         let mut output = "fn main() {\n".to_string();
+        let mut crates = String::new();
+        let mut outer_scope = String::new();
         for cell in cells_vec {
-            if cell.fragment == fragment {
-                output += "\nprintln!(\"\0\");";
-            }
-            output += "\n";
-            output += &cell.contents;
-            if cell.fragment == fragment {
-                output += "\nprintln!(\"\0\");";
+            let lines = cell.contents.split('\n');
+            for line in lines {
+                let line = line.trim();
+                // Don't print if it's not the executing cell
+                if line.starts_with("print") && cell.fragment != fragment {
+                } else if line.starts_with("use") {
+                    outer_scope += line;
+                    outer_scope += "\n";
+                    let (_, full_path) = line.split_once(' ').unwrap();
+                    let (crate_name, _) = full_path.split_once(':').unwrap();
+                    crates += crate_name;
+                    crates += "=\"*\"\n";
+                } else {
+                    output += line;
+                    output += "\n";
+                }
             }
         }
         output += "\n\n}";
+        let mut main = outer_scope.clone();
+        main += &output;
 
         // Get temp file names
         let mut rust_file = String::from(&self.temp_dir);
         let mut cargo_file = String::from(&self.temp_dir);
         rust_file += "/main.rs";
         cargo_file += "/Cargo.toml";
-        let cargo_contents = "[package]\nname = 'output'\nversion = '0.0.1'\nedition = '2021'\n[[bin]]\nname = 'ouput'\npath = 'main.rs'\n";
+        let cargo_contents = format!(
+            "{}\n{}",
+            r#"
+[package]
+name = 'output'
+version = '0.0.1'
+edition = '2021'
+[[bin]]
+name = 'ouput'
+path = 'main.rs'
+[dependencies]
+"#,
+            crates
+        );
 
         // Write the files
-        write(&rust_file, &output).expect("Error writing file");
+        write(&rust_file, &main).expect("Error writing file");
         write(&cargo_file, &cargo_contents).expect("Error writing file");
     }
 
@@ -136,10 +155,6 @@ impl Program {
         }
 
         let output = String::from_utf8(output.stdout).unwrap();
-        let mut x = output.split('\0');
-        x.next();
-        let res = x.next().unwrap();
-        x.next();
-        format!("0\0{}", res)
+        format!("0\0{}", output)
     }
 }
